/*
 * Copyright (c) 2009 - 2018, DHBW Mannheim - TIGERs Mannheim
 */
package edu.tigers.sumatra.botcenter.view;

import java.awt.Color;

import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import edu.tigers.sumatra.botmanager.communication.ENetworkState;


/**
 * Coloured JTextField that uses an ENetworkState.
 *
 * @author AndreR
 */
public class NetStateIndicator extends JTextField
{
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	/**  */
	private static final long	serialVersionUID	= 4127310126348792093L;

	private ENetworkState		state					= ENetworkState.OFFLINE;


	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/** Constructor. */
	public NetStateIndicator()
	{
		setEditable(false);

		setConnectionState(state);
	}


	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------


	// --------------------------------------------------------------------------
	// --- getter/setter --------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * Set connection state.
	 *
	 * @param state
	 */
	public void setConnectionState(final ENetworkState state)
	{
		this.state = state;

		SwingUtilities.invokeLater(() -> {
			switch (state)
			{
				case OFFLINE:
					setText("Offline");
					setBackground(new Color(255, 128, 128));
					break;
				case CONNECTING:
					setText("Connecting");
					setBackground(Color.CYAN);
					break;
				case ONLINE:
					setText("Online");
					setBackground(Color.GREEN);
					break;
				default:
					break;
			}
		});
	}


	/**
	 * @return
	 */
	public ENetworkState getState()
	{
		return state;
	}
}
