/*
 * Copyright (c) 2009 - 2018, DHBW Mannheim - TIGERs Mannheim
 */

package edu.tigers.sumatra.presenter.replay;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import edu.tigers.sumatra.persistence.BerkeleyDb;
import edu.tigers.sumatra.views.ASumatraView;
import edu.tigers.sumatra.wp.IWorldFrameObserver;
import edu.tigers.sumatra.wp.data.BerkeleyCamDetectionFrame;
import edu.tigers.sumatra.wp.data.ExtendedCamDetectionFrame;


public class ReplayCamDetectionController implements IReplayController
{
	private final List<IWorldFrameObserver> wFrameObservers = new CopyOnWriteArrayList<>();
	
	
	public ReplayCamDetectionController(List<ASumatraView> sumatraViews)
	{
		for (ASumatraView view : sumatraViews)
		{
			if (view.getPresenter() instanceof IWorldFrameObserver)
			{
				wFrameObservers.add((IWorldFrameObserver) view.getPresenter());
			}
		}
	}
	
	
	@Override
	public void update(final BerkeleyDb db, final long sumatraTimestampNs)
	{
		BerkeleyCamDetectionFrame camFrame = db.get(BerkeleyCamDetectionFrame.class, sumatraTimestampNs);
		if (camFrame != null)
		{
			ExtendedCamDetectionFrame eFrame = camFrame.getCamFrame();
			for (IWorldFrameObserver vp : wFrameObservers)
			{
				vp.onNewCamDetectionFrame(eFrame);
			}
		}
	}
}
